import { ServiceLogger } from "@effect-aws/commons";
import { Effect, FiberRef, Layer } from "effect";
import { dual } from "effect/Function";
import { globalValue } from "effect/GlobalValue";
/**
 * @since 1.0.0
 * @category api-gateway-v2 service config
 */
const currentApiGatewayV2ServiceConfig = globalValue("@effect-aws/client-api-gateway-v2/currentApiGatewayV2ServiceConfig", () => FiberRef.unsafeMake({}));
/**
 * @since 1.0.0
 * @category api-gateway-v2 service config
 */
export const withApiGatewayV2ServiceConfig = dual(2, (effect, config) => Effect.locally(effect, currentApiGatewayV2ServiceConfig, config));
/**
 * @since 1.0.0
 * @category api-gateway-v2 service config
 */
export const setApiGatewayV2ServiceConfig = (config) => Layer.locallyScoped(currentApiGatewayV2ServiceConfig, config);
/**
 * @since 1.0.0
 * @category adapters
 */
export const toApiGatewayV2ClientConfig = Effect.gen(function* () {
    const { logger: serviceLogger, ...config } = yield* FiberRef.get(currentApiGatewayV2ServiceConfig);
    const logger = serviceLogger === true
        ? yield* ServiceLogger.toClientLogger(ServiceLogger.defaultServiceLogger)
        : (serviceLogger ? yield* ServiceLogger.toClientLogger(ServiceLogger.make(serviceLogger)) : undefined);
    return { logger, ...config };
});
//# sourceMappingURL=ApiGatewayV2ServiceConfig.js.map